"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var URI_1 = require("../util/URI");
var SassImplementation_1 = require("../util/SassImplementation");
var nodeSass = require("node-sass");
var IS_WINDOWS = /win32/.test(require("os").platform());
var normalizeURI = function (_eyeglass, sass) {
    var methods = {
        "eyeglass-uri-preserve($uri)": function ($uri, done) {
            if (!SassImplementation_1.isSassString(sass, $uri)) {
                return done(SassImplementation_1.typeError(sass, "string", $uri));
            }
            var uri = $uri.getValue();
            // decorate the uri
            uri = URI_1.URI.preserve(uri);
            done(sass.types.String(uri));
        },
        "eyeglass-uri-restore($uri)": function ($uri, done) {
            if (!SassImplementation_1.isSassString(sass, $uri)) {
                return done(SassImplementation_1.typeError(sass, "string", $uri));
            }
            var uri = $uri.getValue();
            // restore the uri
            uri = URI_1.URI.restore(uri);
            done(sass.types.String(uri));
        }
    };
    if (IS_WINDOWS || process.env.EYEGLASS_NORMALIZE_PATHS) {
        var $web_1 = nodeSass.types.String("web");
        var $system_1 = nodeSass.types.String("system");
        var egNormalizeUri_1 = function ($uri, $type) {
            if (!SassImplementation_1.isSassString(sass, $type)) {
                throw SassImplementation_1.typeError(sass, "string", $type);
            }
            if (!SassImplementation_1.isSassString(sass, $uri)) {
                throw SassImplementation_1.typeError(sass, "string", $uri);
            }
            var type = $type.getValue();
            var uri = $uri.getValue();
            // normalize the uri for the given type
            uri = URI_1.URI[type](uri);
            return sass.types.String(uri);
        };
        methods["-eyeglass-normalize-uri($uri, $type: web)"] = function ($uri, $type, done) {
            try {
                done(egNormalizeUri_1($uri, $type));
            }
            catch (e) {
                done(e);
            }
        };
        methods["-eyeglass-normalize-assets($assets)"] = function ($assets, done) {
            if (!SassImplementation_1.isSassMap(sass, $assets)) {
                done($assets);
                return;
            }
            var size = $assets.getLength();
            var $newAssets = new sass.types.Map(size);
            for (var i = 0; i < size; i++) {
                var $url = $assets.getKey(i);
                if (SassImplementation_1.isSassString(sass, $url)) {
                    $url = egNormalizeUri_1($url, $web_1);
                }
                $newAssets.setKey(i, $url);
                var $assetProps = $assets.getValue(i);
                if (SassImplementation_1.isSassMap(sass, $assetProps)) {
                    var numAssetProps = $assetProps.getLength();
                    var $newAssetProps = new sass.types.Map(numAssetProps);
                    for (var pi = 0; pi < numAssetProps; pi++) {
                        var $propName = $assetProps.getKey(pi);
                        var $propValue = $assetProps.getValue(pi);
                        $newAssetProps.setKey(pi, $propName);
                        if (SassImplementation_1.isSassString(sass, $propName)) {
                            var propName = $propName.getValue();
                            try {
                                switch (propName) {
                                    case "filepath":
                                        $newAssetProps.setValue(pi, egNormalizeUri_1($propValue, $system_1));
                                        break;
                                    case "uri":
                                        $newAssetProps.setValue(pi, egNormalizeUri_1($propValue, $web_1));
                                        break;
                                    default:
                                        $newAssetProps.setValue(pi, $propValue);
                                }
                            }
                            catch (e) {
                                done(e);
                                return;
                            }
                        }
                        else {
                            $newAssetProps.setValue(pi, $propValue);
                        }
                    }
                    $newAssets.setValue(i, $newAssetProps);
                }
                else {
                    $newAssets.setValue(i, $assetProps);
                }
            }
            done($newAssets);
        };
    }
    return methods;
};
exports.default = normalizeURI;
//# sourceMappingURL=normalize-uri.js.map