"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var path = require("path");
var perf_1 = require("../util/perf");
var ImportUtilities_1 = require("./ImportUtilities");
var FSImporter = function (eyeglass, sass, options, fallbackImporter) {
    var fsURI = /^fs\(([-_a-zA-Z][-_a-zA-Z0-9]+)\)$/;
    return ImportUtilities_1.default.createImporter("fs", function (uri, prev, done) {
        var importUtils = new ImportUtilities_1.default(eyeglass, sass, options, fallbackImporter, this);
        var match = uri.match(fsURI);
        if (match) {
            var identifier = match[1];
            var absolutePath = null;
            if (identifier === "root") {
                absolutePath = options.eyeglass.root;
            }
            else if (!perf_1.existsSync(prev)) {
                absolutePath = path.resolve(".");
            }
            else {
                absolutePath = path.resolve(path.dirname(prev));
            }
            /* istanbul ignore else - TODO: revisit this */
            if (absolutePath) {
                var sassContents = '@import "eyeglass/fs"; @include fs-register-path('
                    + identifier + ', "' + absolutePath + '");';
                var data = {
                    contents: sassContents,
                    file: "fs:" + identifier + ":" + absolutePath
                };
                importUtils.importOnce(data, done);
            }
            else {
                // TODO (test) - how do we get here? needs test case
                done(new Error("Cannot resolve filesystem location of " + prev));
            }
        }
        else {
            importUtils.fallback(uri, prev, done, function () {
                done(null);
            });
        }
    });
};
exports.default = FSImporter;
//# sourceMappingURL=FSImporter.js.map